﻿// #######################################################################################################
// Program Name: CCR Server.exe
// Author: Brice Hilgemann
// Assistance: Professor Blahnik with TcpClient information, MSDN documentation on sockets
// Last Modified: 2/13/2010
// Status: In Progress
// Usage: Creates a Server that has the ability to have multiple clients attached to it in which communcation
//        can occur asynchronously with these and a server can sends information to all of them.
// #######################################################################################################

using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace Server
{
    public partial class Form1 : Form
    {
        Socket ServerSocket;                                     // Socket for the Server
        Socket[] Clients = new Socket[100];                      // Array of ClientStates for multiple clients
        AsyncCallback CallBack;

        byte[] dataholder = new byte[512];                       // Will act as a buffer for our information
        int connection_count = 0;                                // Number of client connections
        int total_connections = 0;                               // Counts total connections made throughout run-time

        int a = 0;    //test

        public Form1()
        {
            InitializeComponent();
        }

        public void Form1_Shown(object sender, EventArgs e)
        // ################################################################################################################
        // Set up defaults at startup when form is first shown
        // ################################################################################################################
        {
            string strHostName = System.Net.Dns.GetHostName();
            IPHostEntry ipEntry = System.Net.Dns.GetHostEntry(strHostName);     // Get current IP Address of system
            IPAddress[] addr = ipEntry.AddressList;

            label5.Text = addr[addr.Length - 1].ToString() + " at Port 10000";
            listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Server started with IP " + addr[addr.Length - 1].ToString() + ":10000");

            listView1.GridLines = true;
            listView1.View = View.Details;
            listView1.Columns.Add("Count", 40, HorizontalAlignment.Center);           //Setup listview
            listView1.Columns[0].TextAlign = HorizontalAlignment.Center;
            listView1.Columns.Add("IP and Port", 125, HorizontalAlignment.Center);
            listView1.Columns[1].TextAlign = HorizontalAlignment.Center;

            textBox1.Enabled = false;
            button1.Enabled = false;

            try
            {
                ServerSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);         // Acts as listener
                IPEndPoint chosen_ip = new IPEndPoint(IPAddress.Any, 10000);                                        // Creates server at default address with port 10000
                ServerSocket.Bind(chosen_ip);                                                                       // Bind the IP Address to listener
                ServerSocket.Listen(5);                                                                             // Create listening queue with queue backlog of 5

                ServerSocket.BeginAccept(new AsyncCallback(HandleClientConnect), null);                             // Set callback to HandleClientConnect if someone tries to connect
            }
            catch (SocketException ex)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue " + ex);         // If something with socket fails
            }

            timer1.Interval = 500;
            timer1.Enabled = true;
        }

        public void HandleClientConnect(IAsyncResult sync_result)
        // ################################################################################################################
        // Connection to client initiated
        //   - New connection attempt has been made, process
        // ################################################################################################################
        {
            try
            {
                Clients[total_connections] = (Socket)ServerSocket.EndAccept(sync_result);               // Process EndAccept and save socket information

                ListViewItem lvi;
                ListViewItem.ListViewSubItem lviSub1;                                                   // Create listview items

                lvi = new ListViewItem(total_connections.ToString());
                lviSub1 = new ListViewItem.ListViewSubItem(lvi, Clients[total_connections].RemoteEndPoint.ToString());     // Add to listview
                lvi.SubItems.Add(lviSub1);

                listView1.Items.Add(lvi);

                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Server connected to " + Clients[total_connections].RemoteEndPoint.ToString());

                HandleDatafromClient(Clients[total_connections]);                                       // Call HandleDatafromClient to set up Callback

                total_connections++;                                                                    // Increase connections
                connection_count++;

                textBox1.Enabled = true;
                button1.Enabled = true;

                label2.Text = "Connections: " + connection_count;

                ServerSocket.BeginAccept(new AsyncCallback(HandleClientConnect), null);                 // Start another callback to listen for more connections
            }
            catch (SocketException ex)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue2 " + ex);         // If something with socket fails
            }
        }

        public void HandleDatafromClient(Socket HolderSocket)
        // ################################################################################################################
        // Information received from Client
        //   - Bytes of data are now in the stream of which are sent from the client, callback was issued and this function
        //     is now called which handles the data
        // ################################################################################################################
        {
            try
            {
                if (CallBack == null)
                {
                    CallBack = new AsyncCallback(HandleDataReceivedfromClient);                         // Set it up to create a callback to HandleDataReceivedfromClient
                }

                HolderSocket.BeginReceive(dataholder, 0, dataholder.Length, SocketFlags.None, CallBack, HolderSocket); // BeginReceive on socket
            }
            catch (SocketException ex)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue3 " + ex);           // If something with socket fails
            }
        }

        public void HandleDataReceivedfromClient(IAsyncResult sync_result)
        // ################################################################################################################
        // Information received from Client
        //   - Bytes of data are now in the stream of which are sent from the client, callback was issued and this function
        //     is now called which handles the data
        // ################################################################################################################
		{
            Socket TempSocket = null;

            try
            {
                TempSocket = (Socket)sync_result.AsyncState;                    // Get information sent in temporary socket

                int received = TempSocket.EndReceive(sync_result);              // Store it in an int to then decode

                if (received > 0)
                {
                    string received_info = Encoding.ASCII.GetString(dataholder, 0, received);    // Convert received to string

                    listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - From Client " + TempSocket.RemoteEndPoint.ToString() + " " + received_info);    //Output string received

                    // ****************************************************************************************************
                    //
                    //
                    //           PROCESS REQUEST FROM CLIENT AND SEND A RESPONSE OF SOMETHING TO THEM
                    //
                    //
                    // ****************************************************************************************************

                    string server_response = "Server Response";
                    byte[] response = Encoding.ASCII.GetBytes(server_response);
                    TempSocket.BeginSend(response, 0, response.Length, SocketFlags.None, new AsyncCallback(ServerSendResponse), TempSocket);

                    listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Sent response of " + server_response + " to Client " + TempSocket.RemoteEndPoint.ToString());
                }
                else
                {
                    CloseSocket(TempSocket);
                }
            }
            catch (ObjectDisposedException)
            {
                CloseSocket(TempSocket);
            }
            catch (SocketException)
            {
                CloseSocket(TempSocket);
            }
            catch (Exception)
            {
                CloseSocket(TempSocket);
            }
		}

        public void ServerSendResponse(IAsyncResult sync_result)
        {
            Socket TempSocket = null;

            try
            {
                TempSocket = (Socket)sync_result.AsyncState;

                int received = TempSocket.EndReceive(sync_result);
            }
            catch (SocketException ex)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue4 " + ex);           // If something with socket fails
                CloseSocket(TempSocket);
            }

            HandleDatafromClient(TempSocket);
        }


        private void CloseSocket(Socket TempSocket)
        {
            for (int i = 0; i < connection_count; i++)
            {
                if (TempSocket.RemoteEndPoint.ToString() == listView1.Items[i].SubItems[1].Text)
                {
                    listView1.Items[i].Remove();

                    listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Client " + TempSocket.RemoteEndPoint.ToString() + " disconnected");

                    TempSocket.Close();
                    TempSocket = null;

                    connection_count--;

                    if (connection_count == 0)
                    {
                        textBox1.Enabled = false;
                        button1.Enabled = false;
                    }

                    label2.Text = "Connections: " + connection_count;

                    return;
                }
            }

        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (ServerSocket != null)
            {
                ServerSocket.Close();
            }
            for(int i=0; i < total_connections; i++)     //Go through connection count, close client sockets
            {
				Clients[i].Close();                     //Close and null it
			}	
        }

        private void button1_Click_1(object sender, EventArgs e)
        {
            try
            {
                byte[] data = Encoding.ASCII.GetBytes(textBox1.Text);

                for (int i = 0; i < connection_count; i++)
                {
                    try
                    {
                        Clients[Convert.ToInt32(listView1.Items[i].SubItems[0].Text)].Send(data);
                    }
                    catch             //Client is no longer there, move up other clients
                    {
                        MessageBox.Show("Client " + listView1.Items[i].SubItems[0].Text + " disconnected?");
                    }
                }
            }
            catch (SocketException ex)
            {
                listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Socket issue5 " + ex);           // If something with socket fails
            }

            listBox1.Items.Add(DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss") + " - Sent " + textBox1.Text + " to Clients");
        }
    }
}
